==> liveshot-1.0.3.tgz: package/src/renderer/megalink/MegalinkCardRenderer.js <==
var inherits = require('inherits');
var CardRenderer = require('../CardRenderer');
var utils = require('./utils');

// --- Constants ---
var MARGINH = 5;
var MARGINV = 3;

var MAX_SHOTS = 10;

var BACKGROUND_COLOR = 'rgb(255, 255, 255)';
var FRONT_COLOR = 'rgb(0, 0, 0)';

var GAUGE_COLOR = 'rgb(0, 0, 0)';
var MARKER_COLOR = 'rgb(0, 255, 0)';
var LAST_MARKER_COLOR = 'rgb(255, 0, 0)';

var TRIANGLE_COLOR = 'rgb(200, 200, 200)';
var TRIANGLE_BORDER_COLOR = 'rgb(150, 150, 150)';

var LANE_NUMBER_BACK_COLOR = 'rgba(64, 64, 64, 0.7)';
var LANE_NUMBER_FRONT_COLOR = 'rgb(245, 245, 245)';

function MegalinkCardRenderer() {
    CardRenderer.prototype.constructor.apply(this);
}

module.exports = MegalinkCardRenderer;
inherits(MegalinkCardRenderer, CardRenderer);

// --- Internal API ---
MegalinkCardRenderer.prototype.initialize = function () {
    CardRenderer.prototype.initialize.apply(this);

    this.style.backColor = BACKGROUND_COLOR;
    this.style.frontColor = FRONT_COLOR;

    this.shotRenderer.setStyle({
        gaugeColor: GAUGE_COLOR,
        markerColor: MARKER_COLOR,
        lastMarkerColor: LAST_MARKER_COLOR
    });

    this.triangleRenderer.setStyle({
        color: TRIANGLE_COLOR,
        borderColor: TRIANGLE_BORDER_COLOR
    });
};

MegalinkCardRenderer.prototype.setTarget = function (targetId) {
    CardRenderer.prototype.setTarget.apply(this, [targetId]);

    this.targetRenderer.setStyle({
        drawFullTarget:true
    });
};

MegalinkCardRenderer.prototype.draw = function () {
    CardRenderer.prototype.draw.apply(this);

    this.drawLaneNumber();
    this.drawHeader();
    this.drawShotList();
    this.drawSums();
};

MegalinkCardRenderer.prototype.drawTarget = function () {
    CardRenderer.prototype.drawTarget.apply(this);

    utils.strokeFrame(this.context, MegalinkCardRenderer.getTargetRect(this.rect), this.style.frontColor);
};

MegalinkCardRenderer.prototype.drawLaneNumber = function () {
    var rect = MegalinkCardRenderer.getLaneNumberRect(this.rect);
    var laneNumber = this.card.lane;

    this.context.fillStyle = LANE_NUMBER_BACK_COLOR;
    this.context.fillRect(rect.x, rect.y, rect.width, rect.height);

    utils.setFont(this.context, laneNumber, .8*rect.width, .8*rect.height);
    this.context.textBaseline = "middle";
    this.context.textAlign = "center";

    this.context.fillStyle = LANE_NUMBER_FRONT_COLOR;
    this.context.fillText(laneNumber, rect.x + rect.width/2, rect.y + rect.height/2);
};

MegalinkCardRenderer.prototype.drawHeader = function () {
    utils.drawFrame(this.context, MegalinkCardRenderer.getHeaderRect(this.rect), this.style.frontColor, this.style.backColor);

    this.drawName();
    this.drawClub();
    this.drawSeriesName();
    this.drawClass();
    this.drawCategory();
};

MegalinkCardRenderer.prototype.drawName = function () {
    var name = this.card.shooter.name;
    var rect = MegalinkCardRenderer.getNameRect(this.rect);

    utils.setFont(this.context, name, rect.width, rect.height);
    this.context.textBaseline = "top";
    this.context.textAlign = "left";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(name, rect.x, rect.y);
};

MegalinkCardRenderer.prototype.drawClub = function () {
    var club = this.card.shooter.club;
    var rect = MegalinkCardRenderer.getClubRect(this.rect);

    utils.setFont(this.context, club, rect.width, rect.height);
    this.context.textBaseline = "bottom";
    this.context.textAlign = "left";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(club, rect.x, rect.y + rect.height);
};

MegalinkCardRenderer.prototype.drawSeriesName = function () {
    var seriesName = this.card.result.seriesName;
    var rect = MegalinkCardRenderer.getSeriesNameRect(this.rect);

    utils.setFont(this.context, seriesName, rect.width, rect.height);
    this.context.textBaseline = "bottom";
    this.context.textAlign = "left";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(seriesName, rect.x, rect.y + rect.height);
};

MegalinkCardRenderer.prototype.drawClass = function () {
    var className = this.card.shooter.className;
    var rect = MegalinkCardRenderer.getClassRect(this.rect);

    utils.setFont(this.context, className, rect.width, rect.height);
    this.context.textBaseline = "bottom";
    this.context.textAlign = "right";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(className, rect.x + rect.width, rect.y + rect.height);
};

MegalinkCardRenderer.prototype.drawCategory = function () {
    var category = this.card.shooter.category;
    var rect = MegalinkCardRenderer.getCategoryRect(this.rect);

    utils.setFont(this.context, category, rect.width, rect.height);
    this.context.textBaseline = "bottom";
    this.context.textAlign = "right";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(category, rect.x + rect.width, rect.y + rect.height);
};

MegalinkCardRenderer.prototype.drawShotList = function () {
    utils.drawFrame(this.context, MegalinkCardRenderer.getShotListRect(this.rect), this.style.frontColor, this.style.backColor);

    this.drawShotListFrame();
    this.drawShotListContent();
};

MegalinkCardRenderer.prototype.drawShotListFrame = function () {
    var rect = MegalinkCardRenderer.getShotListRect(this.rect);

    var numRows = this.getNumRows();
    var numColumns = this.getNumColumns();
    var rowHeight = rect.height / numRows;
    var columnWidth = rect.width / numColumns;

    var pixelRatio = window.devicePixelRatio || 1;

    this.context.strokeStyle = this.style.frontColor;
    this.context.lineWidth = 1*pixelRatio;
    this.context.beginPath();

    for (var i = 1; i < numRows; ++i) {
        var y = rect.y + i*rowHeight;

        this.context.moveTo(rect.x, y);
        this.context.lineTo(rect.x + rect.width, y);
    }

    for (var i = 1; i < numColumns; ++i) {
        var x = rect.x + i*columnWidth;

        this.context.moveTo(x, rect.y);
        this.context.lineTo(x, rect.y + rect.height);
    }

    this.context.stroke();
    this.context.closePath();
};

MegalinkCardRenderer.prototype.drawShotListContent = function () {
    var offset = Math.max(0, this.card.result.shots.length - MAX_SHOTS);
    var shots = this.card.result.shots.slice(-MAX_SHOTS);
    var shotNum = 1;

    for (var idx in shots) {
        var shot = shots[idx];
        this.drawShotContent(shotNum + offset, shot, this.getShotRect(shotNum));

        ++shotNum;
    }
};

MegalinkCardRenderer.prototype.drawShotContent = function (shotNum, shot, rect) {
    utils.setFont(this.context, '10:', .4*rect.width, .9*rect.height);
    this.context.textBaseline = 'middle';
    this.context.fillStyle = this.style.frontColor;

    // draw shot number
    this.context.textAlign = 'left';
    this.context.fillText(shotNum + ':', rect.x + 5, rect.y - rect.height/2);

    // draw shot value
    this.context.textAlign = 'right';
    this.context.fillText(shot.value, rect.x + rect.width - 5, rect.y - rect.height/2);
};

MegalinkCardRenderer.prototype.getNumRows = function () {
    return 5; // XXX set as style property?
};

MegalinkCardRenderer.prototype.getNumColumns = function () {
    var numShots = 0;
    for (var idx in this.card.result.shots) ++numShots;

    return Math.min(2, Math.ceil(numShots / this.getNumRows()));
};

MegalinkCardRenderer.prototype.drawSums = function () {
    var rect = MegalinkCardRenderer.getSumsRect(this.rect);

    utils.drawFrame(this.context, rect, this.style.frontColor, this.style.backColor);

    this.context.fillStyle = this.style.frontColor;
    this.context.fillRect(rect.x + .5*rect.width, rect.y, .5*rect.width, rect.height);

    // draw text
    var seriesSum = this.card.result.seriesSum;
    var totalSum = this.card.result.totalSum;

    utils.setFont(this.context, totalSum, .6*rect.width, .6*rect.height);
    this.context.textBaseline = "middle";
    this.context.textAlign = "center";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(seriesSum, rect.x + .25*rect.width, rect.y + .5*rect.height);

    this.context.fillStyle = this.style.backColor;
    this.context.fillText(totalSum, rect.x + .75*rect.width, rect.y + .5*rect.height);
};

// --- Rect helper methods ---
MegalinkCardRenderer.isPortrait = function (rect) {
    return rect.width / rect.height < 1;
};

MegalinkCardRenderer.prototype.getTargetRect = function () {
    return MegalinkCardRenderer.getTargetRect(this.rect);
};

MegalinkCardRenderer.getTargetRect = function (rect) {
    if (MegalinkCardRenderer.isPortrait(rect)) {
        return {
            x:rect.x,
            y:rect.y + .125*rect.height,
            width:rect.width,
            height:.5*rect.height
        }
    } else {
        return {
            x:rect.x,
            y:rect.y + .25*rect.height,
            width:.6*rect.width,
            height:.75*rect.height
        };
    }
};

MegalinkCardRenderer.getHeaderRect = function (rect) {
    if (MegalinkCardRenderer.isPortrait(rect)) {
        return {
            x:rect.x,
            y:rect.y,
            width:rect.width,
            height:.125*rect.height
        };
    } else {
        return {
            x:rect.x,
            y:rect.y,
            width:rect.width,
            height:.25*rect.height
        };
    }
};

MegalinkCardRenderer.getNameRect = function (rect) {
    var rect = MegalinkCardRenderer.getHeaderRect(rect);

    var width = rect.width;
    var height = .55*rect.height;

    return {
        x:rect.x + MARGINH,
        y:rect.y + MARGINV,
        width:width - 2*MARGINH,
        height:height - 2*MARGINV
    };
};

MegalinkCardRenderer.getClubRect = function (rect) {
    var rect = MegalinkCardRenderer.getHeaderRect(rect);

    var width = .55*rect.width;
    var height = .45*rect.height;

    return {
        x:rect.x + MARGINH,
        y:rect.y + rect.height - height + MARGINV,
        width:width - 3/2*MARGINH,
        height:height - 2*MARGINV
    };
};

MegalinkCardRenderer.getSeriesNameRect = function (rect) {
    var rect = MegalinkCardRenderer.getHeaderRect(rect);

    var width = .25*rect.width;
    var height = .45*rect.height;

    return {
        x:rect.x + .55*rect.width + MARGINH/2,
        y:rect.y + rect.height - height + MARGINV,
        width:width - MARGINH,
        height:height - 2*MARGINV
    };
};

MegalinkCardRenderer.getClassRect = function (rect) {
    var rect = MegalinkCardRenderer.getHeaderRect(rect);

    var width = .13*rect.width;
    var height = .45*rect.height;

    return {
        x:rect.x + .8*rect.width + MARGINH/2,
        y:rect.y + rect.height - height + MARGINV,
        width:width - MARGINH,
        height:height - 2*MARGINV
    };
};

MegalinkCardRenderer.getCategoryRect = function (rect) {
    var rect = MegalinkCardRenderer.getHeaderRect(rect);

    var width = .07*rect.width;
    var height = .45*rect.height;

    return {
        x:rect.x + .93*rect.width + MARGINH/2,
        y:rect.y + rect.height - height + MARGINV,
        width:width - 3/2*MARGINH,
        height:height - 2*MARGINV
    };
};

MegalinkCardRenderer.getShotListRect = function (rect) {
    if (MegalinkCardRenderer.isPortrait(rect)) {
        return {
            x:rect.x,
            y:rect.y + .625*rect.height,
            width:rect.width,
            height:.25*rect.height
        };
    } else {
        return {
            x:rect.x + .6*rect.width,
            y:rect.y + .25*rect.height,
            width:.4*rect.width,
            height:.55*rect.height
        };
    }
};

MegalinkCardRenderer.getLaneNumberRect = function (rect) {
    var targetRect = MegalinkCardRenderer.getTargetRect(rect);
    var size = Math.min(targetRect.width, targetRect.height) / 5;

    targetRect.width = size;
    targetRect.height = size;

    return targetRect;
};

MegalinkCardRenderer.getSumsRect = function (rect) {
    if (MegalinkCardRenderer.isPortrait(rect)) {
        return {
            x:rect.x,
            y:rect.y + .875*rect.height,
            width:rect.width,
            height:.125*rect.height
        };
    } else {
        return {
            x:rect.x + .6*rect.width,
            y:rect.y + .80*rect.height,
            width:.4*rect.width,
            height:.20*rect.height
        };
    }
};

MegalinkCardRenderer.prototype.getShotRect = function (shotNum) {
    var numRows = this.getNumRows();
    var numColumns = this.getNumColumns();
    var rect = MegalinkCardRenderer.getShotListRect(this.rect);

    var cellWidth = rect.width / numColumns;
    var cellHeight = rect.height / numRows;

    var j = Math.floor((shotNum - 1) / numRows);
    var i = shotNum - j*numRows;

    return {
        x:rect.x + j*cellWidth,
        y:rect.y + i*cellHeight,
        width:cellWidth,
        height:cellHeight
    };
};

==> liveshot-1.0.17.tgz: package/src/renderer/megalink/MegalinkCardRenderer.js <==
var inherits = require('inherits');
var CardRenderer = require('../CardRenderer');
var utils = require('./utils');

// --- Constants ---
var MARGINH = 5;
var MARGINV = 3;

var MAX_SHOTS = 10;

var BACKGROUND_COLOR = 'rgb(255, 255, 255)';
var FRONT_COLOR = 'rgb(0, 0, 0)';

var GAUGE_COLOR = 'rgb(0, 0, 0)';
var MARKER_COLOR = 'rgb(0, 255, 0)';
var LAST_MARKER_COLOR = 'rgb(255, 0, 0)';

var TRIANGLE_COLOR = 'rgb(200, 200, 200)';
var TRIANGLE_BORDER_COLOR = 'rgb(150, 150, 150)';

var LANE_NUMBER_BACK_COLOR = 'rgba(64, 64, 64, 0.7)';
var LANE_NUMBER_FRONT_COLOR = 'rgb(245, 245, 245)';

function MegalinkCardRenderer() {
    CardRenderer.prototype.constructor.apply(this);
}

module.exports = MegalinkCardRenderer;
inherits(MegalinkCardRenderer, CardRenderer);

// --- Internal API ---
MegalinkCardRenderer.prototype.initialize = function () {
    CardRenderer.prototype.initialize.apply(this);

    this.style.backColor = BACKGROUND_COLOR;
    this.style.frontColor = FRONT_COLOR;

    this.shotRenderer.setStyle({
        gaugeColor: GAUGE_COLOR,
        markerColor: MARKER_COLOR,
        lastMarkerColor: LAST_MARKER_COLOR
    });

    this.triangleRenderer.setStyle({
        color: TRIANGLE_COLOR,
        borderColor: TRIANGLE_BORDER_COLOR
    });
};

MegalinkCardRenderer.prototype.setTarget = function (targetId) {
    CardRenderer.prototype.setTarget.apply(this, [targetId]);

    this.targetRenderer.setStyle({
        drawFullTarget:true
    });
};

MegalinkCardRenderer.prototype.draw = function () {
    CardRenderer.prototype.draw.apply(this);

    this.drawLaneNumber();
    this.drawHeader();
    this.drawShotList();
    this.drawSums();
};

MegalinkCardRenderer.prototype.drawTarget = function () {
    CardRenderer.prototype.drawTarget.apply(this);

    utils.strokeFrame(this.context, MegalinkCardRenderer.getTargetRect(this.rect), this.style.frontColor);
};

MegalinkCardRenderer.prototype.drawLaneNumber = function () {
    var rect = MegalinkCardRenderer.getLaneNumberRect(this.rect);
    var laneNumber = this.card.lane;

    this.context.fillStyle = LANE_NUMBER_BACK_COLOR;
    this.context.fillRect(rect.x, rect.y, rect.width, rect.height);

    utils.setFont(this.context, laneNumber, .8*rect.width, .8*rect.height);
    this.context.textBaseline = "middle";
    this.context.textAlign = "center";

    this.context.fillStyle = LANE_NUMBER_FRONT_COLOR;
    this.context.fillText(laneNumber, rect.x + rect.width/2, rect.y + rect.height/2);
};

MegalinkCardRenderer.prototype.drawHeader = function () {
    utils.drawFrame(this.context, MegalinkCardRenderer.getHeaderRect(this.rect), this.style.frontColor, this.style.backColor);

    this.drawName();
    this.drawClub();
    this.drawSeriesName();
    this.drawClass();
    this.drawCategory();
};

MegalinkCardRenderer.prototype.drawName = function () {
    var name = this.card.shooter.name;
    var rect = MegalinkCardRenderer.getNameRect(this.rect);

    utils.setFont(this.context, name, rect.width, rect.height);
    this.context.textBaseline = "top";
    this.context.textAlign = "left";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(name, rect.x, rect.y);
};

MegalinkCardRenderer.prototype.drawClub = function () {
    var club = this.card.shooter.club;
    var rect = MegalinkCardRenderer.getClubRect(this.rect);

    utils.setFont(this.context, club, rect.width, rect.height);
    this.context.textBaseline = "bottom";
    this.context.textAlign = "left";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(club, rect.x, rect.y + rect.height);
};

MegalinkCardRenderer.prototype.drawSeriesName = function () {
    var seriesName = this.card.result.seriesName;
    var rect = MegalinkCardRenderer.getSeriesNameRect(this.rect);

    utils.setFont(this.context, seriesName, rect.width, rect.height);
    this.context.textBaseline = "bottom";
    this.context.textAlign = "left";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(seriesName, rect.x, rect.y + rect.height);
};

MegalinkCardRenderer.prototype.drawClass = function () {
    var className = this.card.shooter.className;
    var rect = MegalinkCardRenderer.getClassRect(this.rect);

    utils.setFont(this.context, className, rect.width, rect.height);
    this.context.textBaseline = "bottom";
    this.context.textAlign = "right";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(className, rect.x + rect.width, rect.y + rect.height);
};

MegalinkCardRenderer.prototype.drawCategory = function () {
    var category = this.card.shooter.category;
    var rect = MegalinkCardRenderer.getCategoryRect(this.rect);

    utils.setFont(this.context, category, rect.width, rect.height);
    this.context.textBaseline = "bottom";
    this.context.textAlign = "right";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(category, rect.x + rect.width, rect.y + rect.height);
};

MegalinkCardRenderer.prototype.drawShotList = function () {
    utils.drawFrame(this.context, MegalinkCardRenderer.getShotListRect(this.rect), this.style.frontColor, this.style.backColor);

    this.drawShotListFrame();
    this.drawShotListContent();
};

MegalinkCardRenderer.prototype.drawShotListFrame = function () {
    var rect = MegalinkCardRenderer.getShotListRect(this.rect);

    var numRows = this.getNumRows();
    var numColumns = this.getNumColumns();
    var rowHeight = rect.height / numRows;
    var columnWidth = rect.width / numColumns;

    var pixelRatio = window.devicePixelRatio || 1;

    this.context.strokeStyle = this.style.frontColor;
    this.context.lineWidth = 1*pixelRatio;
    this.context.beginPath();

    for (var i = 1; i < numRows; ++i) {
        var y = rect.y + i*rowHeight;

        this.context.moveTo(rect.x, y);
        this.context.lineTo(rect.x + rect.width, y);
    }

    for (var i = 1; i < numColumns; ++i) {
        var x = rect.x + i*columnWidth;

        this.context.moveTo(x, rect.y);
        this.context.lineTo(x, rect.y + rect.height);
    }

    this.context.stroke();
    this.context.closePath();
};

MegalinkCardRenderer.prototype.drawShotListContent = function () {
    var offset = Math.max(0, this.card.result.shots.length - MAX_SHOTS);
    var shots = this.card.result.shots.slice(-MAX_SHOTS);
    var shotNum = 1;

    for (var idx in shots) {
        var shot = shots[idx];
        this.drawShotContent(shotNum + offset, shot, this.getShotRect(shotNum));

        ++shotNum;
    }
};

MegalinkCardRenderer.prototype.drawShotContent = function (shotNum, shot, rect) {
    utils.setFont(this.context, '*10.9', .55*rect.width, .9*rect.height);
    this.context.textBaseline = 'middle';
    this.context.fillStyle = this.style.frontColor;

    // draw shot number
    this.context.textAlign = 'left';
    this.context.fillText(shotNum + ':', rect.x + 5, rect.y - rect.height/2);

    // draw shot value
    this.context.textAlign = 'right';
    this.context.fillText(shot.value, rect.x + rect.width - 5, rect.y - rect.height/2);
};

MegalinkCardRenderer.prototype.getNumRows = function () {
    return 5; // XXX set as style property?
};

MegalinkCardRenderer.prototype.getNumColumns = function () {
    var numShots = 0;
    for (var idx in this.card.result.shots) ++numShots;

    return Math.min(2, Math.ceil(numShots / this.getNumRows()));
};

MegalinkCardRenderer.prototype.drawSums = function () {
    var rect = MegalinkCardRenderer.getSumsRect(this.rect);

    utils.drawFrame(this.context, rect, this.style.frontColor, this.style.backColor);

    this.context.fillStyle = this.style.frontColor;
    this.context.fillRect(rect.x + .5*rect.width, rect.y, .5*rect.width, rect.height);

    // draw text
    var seriesSum = this.card.result.seriesSum;
    var totalSum = this.card.result.totalSum;

    utils.setFont(this.context, totalSum, .45*rect.width, .6*rect.height);
    this.context.textBaseline = "middle";
    this.context.textAlign = "center";

    this.context.fillStyle = this.style.frontColor;
    this.context.fillText(seriesSum, rect.x + .25*rect.width, rect.y + .5*rect.height);

    this.context.fillStyle = this.style.backColor;
    this.context.fillText(totalSum, rect.x + .75*rect.width, rect.y + .5*rect.height);
};

// --- Rect helper methods ---
MegalinkCardRenderer.isPortrait = function (rect) {
    return rect.width / rect.height < 1;
};

MegalinkCardRenderer.prototype.getTargetRect = function () {
    return MegalinkCardRenderer.getTargetRect(this.rect);
};

MegalinkCardRenderer.getTargetRect = function (rect) {
    if (MegalinkCardRenderer.isPortrait(rect)) {
        return {
            x:rect.x,
            y:rect.y + .125*rect.height,
            width:rect.width,
            height:.5*rect.height
        }
    } else {
        return {
            x:rect.x,
            y:rect.y + .25*rect.height,
            width:.6*rect.width,
            height:.75*rect.height
        };
    }
};

MegalinkCardRenderer.getHeaderRect = function (rect) {
    if (MegalinkCardRenderer.isPortrait(rect)) {
        return {
            x:rect.x,
            y:rect.y,
            width:rect.width,
            height:.125*rect.height
        };
    } else {
        return {
            x:rect.x,
            y:rect.y,
            width:rect.width,
            height:.25*rect.height
        };
    }
};

MegalinkCardRenderer.getNameRect = function (rect) {
    var rect = MegalinkCardRenderer.getHeaderRect(rect);

    var width = rect.width;
    var height = .55*rect.height;

    return {
        x:rect.x + MARGINH,
        y:rect.y + MARGINV,
        width:width - 2*MARGINH,
        height:height - 2*MARGINV
    };
};

MegalinkCardRenderer.getClubRect = function (rect) {
    var rect = MegalinkCardRenderer.getHeaderRect(rect);

    var width = .55*rect.width;
    var height = .45*rect.height;

    return {
        x:rect.x + MARGINH,
        y:rect.y + rect.height - height + MARGINV,
        width:width - 3/2*MARGINH,
        height:height - 2*MARGINV
    };
};

MegalinkCardRenderer.getSeriesNameRect = function (rect) {
    var rect = MegalinkCardRenderer.getHeaderRect(rect);

    var width = .25*rect.width;
    var height = .45*rect.height;

    return {
        x:rect.x + .55*rect.width + MARGINH/2,
        y:rect.y + rect.height - height + MARGINV,
        width:width - MARGINH,
        height:height - 2*MARGINV
    };
};

MegalinkCardRenderer.getClassRect = function (rect) {
    var rect = MegalinkCardRenderer.getHeaderRect(rect);

    var width = .13*rect.width;
    var height = .45*rect.height;

    return {
        x:rect.x + .8*rect.width + MARGINH/2,
        y:rect.y + rect.height - height + MARGINV,
        width:width - MARGINH,
        height:height - 2*MARGINV
    };
};

MegalinkCardRenderer.getCategoryRect = function (rect) {
    var rect = MegalinkCardRenderer.getHeaderRect(rect);

    var width = .07*rect.width;
    var height = .45*rect.height;

    return {
        x:rect.x + .93*rect.width + MARGINH/2,
        y:rect.y + rect.height - height + MARGINV,
        width:width - 3/2*MARGINH,
        height:height - 2*MARGINV
    };
};

MegalinkCardRenderer.getShotListRect = function (rect) {
    if (MegalinkCardRenderer.isPortrait(rect)) {
        return {
            x:rect.x,
            y:rect.y + .625*rect.height,
            width:rect.width,
            height:.25*rect.height
        };
    } else {
        return {
            x:rect.x + .6*rect.width,
            y:rect.y + .25*rect.height,
            width:.4*rect.width,
            height:.55*rect.height
        };
    }
};

MegalinkCardRenderer.getLaneNumberRect = function (rect) {
    var targetRect = MegalinkCardRenderer.getTargetRect(rect);
    var size = Math.min(targetRect.width, targetRect.height) / 5;

    targetRect.width = size;
    targetRect.height = size;

    return targetRect;
};

MegalinkCardRenderer.getSumsRect = function (rect) {
    if (MegalinkCardRenderer.isPortrait(rect)) {
        return {
            x:rect.x,
            y:rect.y + .875*rect.height,
            width:rect.width,
            height:.125*rect.height
        };
    } else {
        return {
            x:rect.x + .6*rect.width,
            y:rect.y + .80*rect.height,
            width:.4*rect.width,
            height:.20*rect.height
        };
    }
};

MegalinkCardRenderer.prototype.getShotRect = function (shotNum) {
    var numRows = this.getNumRows();
    var numColumns = this.getNumColumns();
    var rect = MegalinkCardRenderer.getShotListRect(this.rect);

    var cellWidth = rect.width / numColumns;
    var cellHeight = rect.height / numRows;

    var j = Math.floor((shotNum - 1) / numRows);
    var i = shotNum - j*numRows;

    return {
        x:rect.x + j*cellWidth,
        y:rect.y + i*cellHeight,
        width:cellWidth,
        height:cellHeight
    };
};
